# runs iterative resynthesis: the LUT-mapping script "strash; dch; if; mfs" is repeatedly executed
#
# usage: tclsh resynth.tcl src_file dest_file iterations
#            src_file - source file (circuit)
#            dest_file - target file
#            iterations - number of iterations

source abctcl.tcl

# initialize ABC

global timeout
set timeout 3600

if { [catch { abc::init res }] } {
    puts $errorCode
    puts ".failed"
    foreach initFile $res(sourced) {
        puts $initFile
    }
    exit
}

set file1 [lindex $argv 0]
set file2 [lindex $argv 1]
set reps [lindex $argv 2]

# read source
if { [catch { abc::read res $file1 }] } {
		puts "Reading $file1 failed"
		exit
}

# run resynthesis
for {set i 0} {$i < $reps} {incr i} {
	if { [catch { abc::command "strash; dch; if; mfs" res }] } {
		puts "Synthesis failed"
	} else {
		puts $res(rawOutput)
	}
}

# write result
if { [catch { abc::command "write_blif $file2" res }] } {
		puts "Writing $file2 failed"
		exit
}

abc::done
