# runs iterative resynthesis: the standard cells-mapping script "strash; map" is repeatedly executed, until a given result quality is reached
#
# usage: tclsh resynth.tcl src_file dest_file iterations
#            src_file - source file (circuit)
#            dest_file - target file
#            gates - the required number of gates

source abctcl.tcl

# initialize ABC

global timeout
set timeout 3600

if { [catch { abc::init res }] } {
    puts $errorCode
    puts ".failed"
    foreach initFile $res(sourced) {
        puts $initFile
    }
    exit
}

set file1 [lindex $argv 0]
set file2 [lindex $argv 1]
set gates [lindex $argv 2]

# read source
if { [catch { abc::read res $file1 }] } {
		puts "Reading $file1 failed"
		exit
}

# read library
	if { [catch { abc::command "read_library mcnc.genlib" res }] } {
		puts "Reading library failed"
		exit
}

# run resynthesis
set g 100000
while {$g > $gates} {
	if { [catch { abc::command "strash; map" res }] } {
		puts "Synthesis failed"
	} else {
		abc::print_stats res
		puts $res(nd)
		set g $res(nd)
	}
}

# write result
if { [catch { abc::command "write_blif $file2" res }] } {
		puts "Writing $file2 failed"
		exit
}

abc::done
