# abctcl sample program



# include abctcl.tcl script
# relative scripts JS
set thisDir [file dirname [info script]]
source [file join $thisDir abctcl.ng.tcl]


# compares several resynthesis commands
# calls each given command several times until number of ANDs stops differing
# prints name, number of ANDs and number of iterations for command with best
# result - minimal number of ANDs
proc compareResyn {filename} {
	# data file name
	# set filename {examples/apex4.pla}
	# set filename {examples/pj1.blif}
	# some resynthesis commands may take longer time to execute
	# timeout variable is defined on global level
	global timeout
	set timeout 60

	# start value - +infinity
	set startBestAND 1000000
	# best and value from all synthesis commands
	set globalBestAND $startBestAND
	# name of best synthesis command
	set globalBestSynt "not available"
	# in how many iterations was value of globalBestAND reached
	set globalBestRunsNeeded -1

	# resynthesis commands to compare
	set resynNames {"resyn; resyn2" "resyn2" "resyn" "resyn2a" "resyn3"}

	foreach resynName $resynNames {
		puts -nonewline "\nread: "

		# read data
		abc::read readRes $filename
        puts "ok"
		puts [format "'%s'" $resynName]
		puts [format "globalBestAND: %d, globalBestSynt: '%s', globalBestRunsNeeded: %d" $globalBestAND $globalBestSynt $globalBestRunsNeeded]

		set bestSynt $resynName
		# local best number of ands
		set bestAND $startBestAND

		set oldBestAND [expr $bestAND + 1]
		set runsNeeded 0

		# loop with one command
		while {$oldBestAND > $bestAND} {
			set oldBestAND $bestAND

			incr runsNeeded

			# call synthesis command
			abc::command $resynName anyCommandResult

			# call print_stats
			abc::print_stats print_statsResult

			# use only 'and' field from output
			set bestAND $print_statsResult(and)
			if {$bestAND < $globalBestAND} {
				set globalBestAND $bestAND
				set globalBestSynt $resynName
				set globalBestRunsNeeded $runsNeeded
			}

			puts [format "localBestAND: %d, globalBestAND: %d, globalBestSynt: '%s', globalBestRunsNeeded: %d" $bestAND $globalBestAND $globalBestSynt $globalBestRunsNeeded]
		}	
	}
	
	puts "done"
	puts [format "globalBestAND: %d" $globalBestAND]
	puts [format "globalBestSynt: %s" $globalBestSynt]
	puts [format "globalBestRunsNeeded: %s" $globalBestRunsNeeded]

	return 1
}

# ---- main

if { [llength $argv] == 0} {
    puts "usage: $argv0 filename"
    exit 1
}

puts ".start"

# init
puts -nonewline "abc::init ... "
if { [catch { abc::init res }] } {
    puts ""
    puts $errorCode
    puts ".failed"
    foreach initFile $res(sourced) {
        puts $initFile
    }
} else {
    foreach initFile $res(sourced) {
        puts -nonewline "$initFile ... "
    }
    puts ".ok"
    puts $res(abcVersion)
}
array unset res

if { ![compareResyn [lindex $argv 0]] } {
    puts $errorCode
}

abc::done

puts ".done"

